@@ -43,8 +43,8 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <syslog.h>
-#include <arpa/inet.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <errno.h>
 #include <string.h>
 
@@ -58,6 +58,7 @@
 #include "nfnet.h"
 #include "nf_common.h"
 #include "bookkeeper.h"
+#include "nfxstat.h"
 #include "collector.h"
 #include "netflow_v5_v7.h"
 
@@ -74,19 +75,14 @@ extern uint32_t overwrite_sampling;
 
 /* module limited globals */
 static extension_info_t v5_extension_info;		// common for all v5 records
-static uint16_t v5_output_record_size;
+static uint16_t v5_output_record_size, v5_output_record_base_size;
 
 // All required extension to save full v5 records
 static uint16_t v5_full_mapp[] = { EX_IO_SNMP_2, EX_AS_2, EX_MULIPLE, EX_NEXT_HOP_v4, EX_ROUTER_IP_v4, EX_ROUTER_ID, 0 };
 
-typedef struct v5_block_s {
-	uint32_t	srcaddr;
-	uint32_t	dstaddr;
-	uint32_t	dPkts;
-	uint32_t	dOctets;
-	uint32_t	data[1];	// link to next record
-} v5_block_t;
-#define V5_BLOCK_DATA_SIZE (sizeof(v5_block_t) - sizeof(uint32_t))
+// to simplify, assume blocks with 64 bit counters to check for enough buffer space
+// regardless if 32 or 64 bit packet/byte counters
+#define V5_BLOCK_DATA_SIZE (sizeof(ipv4_block_t) - sizeof(uint32_t) + 2 * sizeof(uint64_t))
 
 typedef struct exporter_v5_s {
 	// link chain
@@ -145,8 +141,8 @@ uint16_t	map_size;
 		i++;
 	}
 	// extension_size contains the sum of all optional extensions
-	// caculate the record size 
-	v5_output_record_size = COMMON_RECORD_DATA_SIZE + V5_BLOCK_DATA_SIZE + extension_size;	
+	// caculate the record size without counters!
+	v5_output_record_base_size = COMMON_RECORD_DATA_SIZE + 8 + extension_size;	// + 8 for 2 x IPv4 addr
 
 	// now the full extension map size
 	map_size 	+= sizeof(extension_map_t);
@@ -251,14 +247,14 @@ char ipstr[IP_STRING_LEN];
 		strncpy(ipstr, "<unknown>", IP_STRING_LEN);
 	}
 
-	dbg_printf("New Exporter: v5 Extension ID: %i, IP: %s, Sampling Mode: %i, Sampling Interval: %i\n", 
+	dbg_printf("New Exporter: v5 Extension ID: %i, IP: %s, Sampling Mode: %i, Sampling Interval: %u\n", 
 		(*e)->extension_map->map_id, ipstr, (*e)->sampling_mode	,(*e)->sampling_interval);
-	syslog(LOG_INFO, "Process_v5: New exporter: engine id %u, type %u, IP: %s, Sampling Mode: %i, Sampling Interval: %i\n", 
+	syslog(LOG_INFO, "Process_v5: New exporter: engine id %u, type %u, IP: %s, Sampling Mode: %i, Sampling Interval: %u\n", 
 		( engine_tag & 0xFF ),  ( (engine_tag >> 8) & 0xFF ), ipstr, (*e)->sampling_mode	,(*e)->sampling_interval         );
 
 	if ( overwrite_sampling > 0 )  {
 		(*e)->sampling_interval = overwrite_sampling;
-		syslog(LOG_INFO, "Process_v5: Hard overwrite sampling rate: %llu\n", (*e)->sampling_interval);
+		syslog(LOG_INFO, "Process_v5: Hard overwrite sampling rate: %u\n", (*e)->sampling_interval);
 	}
 
 	return (*e);
@@ -291,7 +287,19 @@ char		*string;
 			syslog(LOG_ERR,"Process_v5: Exporter NULL: Abort v5/v7 record processing");
 			return;
 		}
-		flags = exporter->sampling_interval == 1 ? 0 : FLAG_SAMPLED;
+
+		// calculate record size depending on counter size
+		// sigh .. one day I should fix switch to 64bits
+		if ( exporter->sampling_interval == 1 ) {
+			flags = 0;
+			v5_output_record_size = v5_output_record_base_size + 8; // 2 x  4 byte counters
+		} else {
+			flags = 0;
+			SetFlag(flags, FLAG_SAMPLED);
+			SetFlag(flags, FLAG_PKG_64);
+			SetFlag(flags, FLAG_BYTES_64);
+			v5_output_record_size = v5_output_record_base_size + 16; // 2 x  8 byte counters
+		}
 
 		extension_map = exporter->extension_map;
 
@@ -301,10 +309,10 @@ char		*string;
 		// this many data to process
 		size_left	= in_buff_cnt;
 
-		common_record = fs->nffile->writeto;
+		common_record = fs->nffile->buff_ptr;
 		done = 0;
 		while ( !done ) {
-			v5_block_t			*v5_block;
+			ipv4_block_t	*ipv4_block;
 
 			/* Process header */
 	
@@ -312,14 +320,14 @@ char		*string;
 	  		count	= ntohs(v5_header->count);
 			if ( count > NETFLOW_V5_MAX_RECORDS ) {
 				syslog(LOG_ERR,"Process_v5: Unexpected record count in header: %i. Abort v5/v7 record processing", count);
-				fs->nffile->writeto = (void *)common_record;
+				fs->nffile->buff_ptr = (void *)common_record;
 				return;
 			}
 
 			// input buffer size check for all expected records
 			if ( size_left < ( NETFLOW_V5_HEADER_LENGTH + count * flow_record_length) ) {
 				syslog(LOG_ERR,"Process_v5: Not enough data to process v5 record. Abort v5/v7 record processing");
-				fs->nffile->writeto = (void *)common_record;
+				fs->nffile->buff_ptr = (void *)common_record;
 				return;
 			}
 	
@@ -331,8 +339,8 @@ char		*string;
 			}
 
 			// map output record to memory buffer
-			common_record	= (common_record_t *)fs->nffile->writeto;
-			v5_block		= (v5_block_t *)common_record->data;
+			common_record	= (common_record_t *)fs->nffile->buff_ptr;
+			ipv4_block		= (ipv4_block_t *)common_record->data;
 
 			// sequence check
 			if ( exporter->first ) {
@@ -349,12 +357,12 @@ char		*string;
 				}
 				if (exporter->distance != exporter->last_count) {
 #define delta(a,b) ( (a)>(b) ? (a)-(b) : (b)-(a) )
-					fs->stat_record.sequence_failure++;
+					fs->nffile->stat_record->sequence_failure++;
 					/*
 					syslog(LOG_ERR,"Flow v%d sequence last:%llu now:%llu mismatch. Missing: dist:%lu flows",
 						version, exporter->last_sequence, exporter->sequence, exporter->distance);
-
 					*/
+
 				}
 			}
 			exporter->last_count  = count;
@@ -373,6 +381,7 @@ char		*string;
 			/* loop over each records associated with this header */
 			for (i = 0; i < count; i++) {
 				pointer_addr_t	bsize;
+				uint64_t	packets, bytes;
 				void	*data_ptr;
 				uint8_t *s1, *s2;
 				int j, id;
@@ -392,13 +401,41 @@ char		*string;
 	  			common_record->fwd_status 	= 0;
 
 				// v5 typed data as fixed struct v5_block
-	  			v5_block->srcaddr	= ntohl(v5_record->srcaddr);
-	  			v5_block->dstaddr	= ntohl(v5_record->dstaddr);
-	  			v5_block->dPkts  	= ntohl(v5_record->dPkts);
-	  			v5_block->dOctets	= ntohl(v5_record->dOctets);
+	  			ipv4_block->srcaddr	= ntohl(v5_record->srcaddr);
+	  			ipv4_block->dstaddr	= ntohl(v5_record->dstaddr);
+
+				if ( exporter->sampling_interval == 1 ) {
+					value32_t   *v = (value32_t *)ipv4_block->data;
+
+	  				packets  	= (uint64_t)ntohl(v5_record->dPkts);
+	  				bytes		= (uint64_t)ntohl(v5_record->dOctets);
+
+					v->val		= packets;
+					v 			= (value32_t *)v->data;
+					v->val		= bytes;
+
+					data_ptr = (void *)v->data;
+				} else {
+					value64_t   *v = (value64_t *)ipv4_block->data;
+					uint32_t    *ptr = (uint32_t *)&packets;
+
+	  				packets  	= (uint64_t)ntohl(v5_record->dPkts)   * (uint64_t)exporter->sampling_interval;
+	  				bytes		= (uint64_t)ntohl(v5_record->dOctets) * (uint64_t)exporter->sampling_interval;
+
+					// pack packets in 32bit chunks
+					v->val.val32[0] = ptr[0];
+					v->val.val32[1] = ptr[1];
+
+					// pack bytes in 32bit chunks
+					v   = (value64_t *)v->data;
+					ptr = (uint32_t *)&bytes;
+					v->val.val32[0] = ptr[0];
+					v->val.val32[1] = ptr[1];
+
+					data_ptr = (void *)v->data;
+				}
 
 				// process optional extensions
-				data_ptr = (void *)v5_block->data;
 				j = 0;
 				while ( (id = extension_map->ex_id[j]) != 0 ) {
 					switch (id) {
@@ -451,9 +488,13 @@ char		*string;
 	  			First	 				= ntohl(v5_record->First);
 	  			Last		 			= ntohl(v5_record->Last);
 
-
 #ifdef FIXTIMEBUG
-				// assume bug, some users reported - swap time stamps
+				/* 
+				 * Some users report, that they see flows, which have duration time of about 40days
+				 * which is almost the overflow value. Investigating this, it cannot be an overflow
+				 * and the difference is always 15160 or 15176 msec too little for a classical 
+				 * overflow. Therefore assume this must be an exporter bug
+				 */
 				if ( First > Last && ( (First - Last)  < 20000) ) {
 					uint32_t _t;
 					syslog(LOG_ERR,"Process_v5: Unexpected time swap: First 0x%llx smaller than boot time: 0x%llx", start_time, boot_time);
@@ -465,9 +506,10 @@ char		*string;
 				if ( First > Last ) {
 					/* First in msec, in case of msec overflow, between start and end */
 					start_time = boot_time - 0x100000000LL + (uint64_t)First;
-				} else
-					start_time = (uint64_t)First + boot_time;
-		
+				} else {
+					start_time = boot_time + (uint64_t)First;
+				}
+
 				/* end time in msecs */
 				end_time = (uint64_t)Last + boot_time;
 
@@ -491,13 +533,11 @@ char		*string;
 	
 	
 				// Update stats
-				v5_block->dPkts   *= exporter->sampling_interval;
-				v5_block->dOctets *= exporter->sampling_interval;
 				switch (common_record->prot) {
 					case IPPROTO_ICMP:
-						fs->stat_record.numflows_icmp++;
-						fs->stat_record.numpackets_icmp += v5_block->dPkts;
-						fs->stat_record.numbytes_icmp   += v5_block->dOctets;
+						fs->nffile->stat_record->numflows_icmp++;
+						fs->nffile->stat_record->numpackets_icmp += packets;
+						fs->nffile->stat_record->numbytes_icmp   += bytes;
 						// fix odd CISCO behaviour for ICMP port/type in src port
 						if ( common_record->srcport != 0 ) {
 							s1 = (uint8_t *)&(common_record->srcport);
@@ -508,24 +548,48 @@ char		*string;
 						}
 						break;
 					case IPPROTO_TCP:
-						fs->stat_record.numflows_tcp++;
-						fs->stat_record.numpackets_tcp += v5_block->dPkts;
-						fs->stat_record.numbytes_tcp   += v5_block->dOctets;
+						fs->nffile->stat_record->numflows_tcp++;
+						fs->nffile->stat_record->numpackets_tcp += packets;
+						fs->nffile->stat_record->numbytes_tcp   += bytes;
 						break;
 					case IPPROTO_UDP:
-						fs->stat_record.numflows_udp++;
-						fs->stat_record.numpackets_udp += v5_block->dPkts;
-						fs->stat_record.numbytes_udp   += v5_block->dOctets;
+						fs->nffile->stat_record->numflows_udp++;
+						fs->nffile->stat_record->numpackets_udp += packets;
+						fs->nffile->stat_record->numbytes_udp   += bytes;
 						break;
 					default:
-						fs->stat_record.numflows_other++;
-						fs->stat_record.numpackets_other += v5_block->dPkts;
-						fs->stat_record.numbytes_other   += v5_block->dOctets;
+						fs->nffile->stat_record->numflows_other++;
+						fs->nffile->stat_record->numpackets_other += packets;
+						fs->nffile->stat_record->numbytes_other   += bytes;
 				}
-				fs->stat_record.numflows++;
-				fs->stat_record.numpackets	+= v5_block->dPkts;
-				fs->stat_record.numbytes	+= v5_block->dOctets;
-	
+				fs->nffile->stat_record->numflows++;
+				fs->nffile->stat_record->numpackets	+= packets;
+				fs->nffile->stat_record->numbytes	+= bytes;
+
+				if ( fs->xstat ) {
+					uint32_t bpp = packets ? (bytes/packets) : 0;
+					if ( bpp > MAX_BPP ) 
+						bpp = MAX_BPP;
+					if ( common_record->prot == IPPROTO_TCP ) {
+						fs->xstat->bpp_histogram->tcp.bpp[bpp]++;
+						fs->xstat->bpp_histogram->tcp.count++;
+
+						fs->xstat->port_histogram->src_tcp.port[common_record->srcport]++;
+						fs->xstat->port_histogram->dst_tcp.port[common_record->dstport]++;
+						fs->xstat->port_histogram->src_tcp.count++;
+						fs->xstat->port_histogram->dst_tcp.count++;
+					} else if ( common_record->prot == IPPROTO_UDP ) {
+						fs->xstat->bpp_histogram->udp.bpp[bpp]++;
+						fs->xstat->bpp_histogram->udp.count++;
+
+						fs->xstat->port_histogram->src_udp.port[common_record->srcport]++;
+						fs->xstat->port_histogram->dst_udp.port[common_record->dstport]++;
+						fs->xstat->port_histogram->src_udp.count++;
+						fs->xstat->port_histogram->dst_udp.count++;
+					}
+				}
+
+
 				if ( verbose ) {
 					master_record_t master_record;
 					ExpandRecord_v2((common_record_t *)common_record, &v5_extension_info, &master_record);
@@ -537,23 +601,24 @@ char		*string;
 				v5_record		= (netflow_v5_record_t *)((pointer_addr_t)v5_record + flow_record_length);
 
 				if ( ((pointer_addr_t)data_ptr - (pointer_addr_t)common_record) != v5_output_record_size ) {
-					printf("Panic size check: ptr diff: %llu, record size: %u\n", ((pointer_addr_t)data_ptr - (pointer_addr_t)common_record), v5_output_record_size ); 
+					printf("Panic size check: ptr diff: %llu, record size: %u\n", 
+						(unsigned long long)((pointer_addr_t)data_ptr - (pointer_addr_t)common_record), v5_output_record_size ); 
 					abort();
 				}
 				// advance to next output record
 				common_record	= (common_record_t *)data_ptr;
-				v5_block		= (v5_block_t *)common_record->data;
+				ipv4_block		= (ipv4_block_t *)common_record->data;
 				
 				// buffer size sanity check - should never happen, but check it anyway
 				bsize = (pointer_addr_t)common_record - (pointer_addr_t)fs->nffile->block_header - sizeof(data_block_header_t);
 				if ( bsize >= BUFFSIZE ) {
 					syslog(LOG_ERR,"### Software error ###: %s line %d", __FILE__, __LINE__);
 					syslog(LOG_ERR,"Process_v5: Output buffer overflow! Flush buffer and skip records.");
-					syslog(LOG_ERR,"Buffer size: size: %u, bsize: %u > %u", fs->nffile->block_header->size, bsize, BUFFSIZE);
+					syslog(LOG_ERR,"Buffer size: size: %u, bsize: %llu > %u", fs->nffile->block_header->size, (unsigned long long)bsize, BUFFSIZE);
 					// reset buffer
 					fs->nffile->block_header->size 		= 0;
 					fs->nffile->block_header->NumRecords = 0;
-					fs->nffile->writeto = (void *)((pointer_addr_t)fs->nffile->block_header + sizeof(data_block_header_t) );
+					fs->nffile->buff_ptr = (void *)((pointer_addr_t)fs->nffile->block_header + sizeof(data_block_header_t) );
 					return;
 				}
 
@@ -562,7 +627,7 @@ char		*string;
 		// update file record size ( -> output buffer size )
 		fs->nffile->block_header->NumRecords	+= count;
 		fs->nffile->block_header->size 		+= count * v5_output_record_size;
-		fs->nffile->writeto 					= (void *)common_record;
+		fs->nffile->buff_ptr 					= (void *)common_record;
 
 		// still to go for this many input bytes
 		size_left 	-= NETFLOW_V5_HEADER_LENGTH + count * flow_record_length;
@@ -617,7 +682,7 @@ uint32_t	i, id, t1, t2;
 		output_engine.first 	 = 0;
 	}
 	if ( cnt == 0 ) {
-		peer->writeto  = (void *)((pointer_addr_t)peer->send_buffer + NETFLOW_V5_HEADER_LENGTH);
+		peer->buff_ptr  = (void *)((pointer_addr_t)peer->send_buffer + NETFLOW_V5_HEADER_LENGTH);
 		v5_output_record = (netflow_v5_record_t *)((pointer_addr_t)v5_output_header + (pointer_addr_t)sizeof(netflow_v5_header_t));	
 		output_engine.sequence = output_engine.last_sequence + output_engine.last_count;
 		v5_output_header->flow_sequence	= htonl(output_engine.sequence);
@@ -677,7 +742,7 @@ uint32_t	i, id, t1, t2;
 	cnt++;
 
 	v5_output_header->count 	= htons(cnt);
-	peer->writeto = (void *)((pointer_addr_t)peer->writeto + NETFLOW_V5_RECORD_LENGTH);
+	peer->buff_ptr = (void *)((pointer_addr_t)peer->buff_ptr + NETFLOW_V5_RECORD_LENGTH);
 	v5_output_record++;
 	if ( cnt == NETFLOW_V5_MAX_RECORDS ) {
 		peer->flush = 1;
