@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2011, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -60,13 +60,12 @@
 
 /* global vars */
 
+// required for idet filter in nftree.c
 char 	*CurrentIdent;
 
-/* local vars */
-static file_header_t	FileHeader;
-static stat_record_t	NetflowStat;
 
-#define file_compressed (FileHeader.flags & FLAG_COMPRESSED)
+#define READ_FILE	1
+#define WRITE_FILE	1
 
 // LZO params
 #define LZO_BUFFSIZE  ((BUFFSIZE + BUFFSIZE / 16 + 64 + 3) + sizeof(data_block_header_t))
@@ -85,25 +84,10 @@ static int LZO_initialize(void);
 extern char *nf_error;
 
 /* function prototypes */
-
-static void ZeroStat(void);
+static nffile_t *NewFile(void);
 
 /* function definitions */
 
-static void ZeroStat() {
-
-	FileHeader.NumBlocks 	= 0;
-	strncpy(FileHeader.ident, IdentNone, IdentLen);
-
-	NetflowStat.first_seen  = 0;
-	NetflowStat.last_seen	= 0;
-	NetflowStat.msec_first	= 0;
-	NetflowStat.msec_last	= 0;
-
-	CurrentIdent			= FileHeader.ident;
-
-} // End of ZeroStat
-
 void SumStatRecords(stat_record_t *s1, stat_record_t *s2) {
 
 	s1->numflows			+= s2->numflows;
@@ -142,32 +126,17 @@ void SumStatRecords(stat_record_t *s1, stat_record_t *s2) {
 } // End of SumStatRecords
 
 
-char *GetIdent(void) {
-
-	return CurrentIdent;
-
-} // End of GetIdent
-
-int IsCompressed(void) {
-	return file_compressed;
-} // End of IsCompressed
-
-int IsAnonymized(void) {
-	return (FileHeader.flags & FLAG_ANONYMIZED);
-} // End of IsCompressed
-
 static int LZO_initialize(void) {
 
 	if (lzo_init() != LZO_E_OK) {
 			// this usually indicates a compiler bug - try recompiling 
 			// without optimizations, and enable `-DLZO_DEBUG' for diagnostics
-			snprintf(error_string, ERR_SIZE,"Compression lzo_init() failed.\n");
+			LogError("Compression lzo_init() failed.\n");
 			return 0;
 	} 
 	lzo_buff = malloc(BUFFSIZE+ sizeof(data_block_header_t));
 	if ( !lzo_buff ) {
-		snprintf(error_string, ERR_SIZE, "malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		error_string[ERR_SIZE-1] = 0;
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		return 0;
 	}
 	lzo_initialized = 1;
@@ -177,170 +146,158 @@ static int LZO_initialize(void) {
 } // End of LZO_initialize
 
 
-int OpenFile(char *filename, stat_record_t **stat_record, char **err){
+nffile_t *OpenFile(char *filename, nffile_t *nffile){
 struct stat stat_buf;
-int fd, ret;
+int ret, allocated;
+
+	if ( !nffile ) {
+		nffile = NewFile();
+		if ( nffile == NULL ) {
+			return NULL;
+		}
+		allocated = 1;
+	} else 
+		allocated = 0;
 
-	*err = NULL;
-	if ( stat_record ) 
-		*stat_record = &NetflowStat;
 
 	if ( filename == NULL ) {
 		// stdin
-		ZeroStat();
-		fd = STDIN_FILENO;
+		// Zero Stat
+		nffile->fd = STDIN_FILENO;
 	} else {
 		// regular file
 		if ( stat(filename, &stat_buf) ) {
-			snprintf(error_string, ERR_SIZE, "Can't stat '%s': %s\n", filename, strerror(errno));
-			error_string[ERR_SIZE-1] = 0;
-			*err = error_string;
-			ZeroStat();
-			return -1;
+			LogError("Can't stat '%s': %s\n", filename, strerror(errno));
+			if ( allocated ) {
+				DisposeFile(nffile);
+				return NULL;
+			}
 		}
 
 		if (!S_ISREG(stat_buf.st_mode) ) {
-			snprintf(error_string, ERR_SIZE, "'%s' is not a file\n", filename);
-			error_string[ERR_SIZE-1] = 0;
-			*err = error_string;
-			ZeroStat();
-			return -1;
+			LogError("'%s' is not a file\n", filename);
+			if ( allocated ) {
+				DisposeFile(nffile);
+				return NULL;
+			}
 		}
 
 		// printf("Statfile %s\n",filename);
-		fd =  open(filename, O_RDONLY);
-		if ( fd < 0 ) {
-			snprintf(error_string, ERR_SIZE, "Error open file: %s\n", strerror(errno));
-			error_string[ERR_SIZE-1] = 0;
-			*err = error_string;
-			ZeroStat();
-			return fd;
+		nffile->fd = open(filename, O_RDONLY);
+		if ( nffile->fd < 0 ) {
+			LogError("Error open file: %s\n", strerror(errno));
+			if ( allocated ) {
+				DisposeFile(nffile);
+				return NULL;
+			}
 		}
 
 	}
 
-	ret = read(fd, (void *)&FileHeader, sizeof(FileHeader));
-	if ( FileHeader.magic != MAGIC ) {
-		snprintf(error_string, ERR_SIZE, "Open file '%s': bad magic: 0x%X\n", filename ? filename : "<stdin>", FileHeader.magic );
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
-		ZeroStat();
-		close(fd);
-		return -1;
+	ret = read(nffile->fd, (void *)nffile->file_header, sizeof(file_header_t));
+	if ( nffile->file_header->magic != MAGIC ) {
+		LogError("Open file '%s': bad magic: 0x%X\n", filename ? filename : "<stdin>", nffile->file_header->magic );
+		CloseFile(nffile);
+		if ( allocated ) {
+			DisposeFile(nffile);
+			return NULL;
+		}
 	}
-	if ( FileHeader.version != LAYOUT_VERSION_1 ) {
-		snprintf(error_string, ERR_SIZE,"Open file %s: bad version: %u\n", filename, FileHeader.version );
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
-		ZeroStat();
-		close(fd);
-		return -1;
+
+	if ( nffile->file_header->version != LAYOUT_VERSION_1 ) {
+		LogError("Open file %s: bad version: %u\n", filename, nffile->file_header->version );
+		CloseFile(nffile);
+		if ( allocated ) {
+			DisposeFile(nffile);
+			return NULL;
+		}
 	}
-	read(fd, (void *)&NetflowStat, sizeof(NetflowStat));
 
-// for debugging:
-/*
-	printf("Magic: 0x%X\n", FileHeader.magic);
-	printf("Version: %i\n", FileHeader.version);
-	printf("Flags: %i\n", FileHeader.flags);
-	printf("NumBlocks: %i\n", FileHeader.NumBlocks);
-	printf("Ident: %s\n\n", FileHeader.ident);
+	ret = read(nffile->fd, (void *)nffile->stat_record, sizeof(stat_record_t));
+	if ( ret < 0 ) {
+		LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		CloseFile(nffile);
+		if ( allocated ) {
+			DisposeFile(nffile);
+			return NULL;
+		}
+	}
 
-	printf("Flows: %llu\n", NetflowStat.numflows);
-	printf("Flows_tcp: %llu\n", NetflowStat.numflows_tcp);
-	printf("Flows_udp: %llu\n", NetflowStat.numflows_udp);
-	printf("Flows_icmp: %llu\n", NetflowStat.numflows_icmp);
-	printf("Flows_other: %llu\n", NetflowStat.numflows_other);
-	printf("Packets: %llu\n", NetflowStat.numpackets);
-	printf("Packets_tcp: %llu\n", NetflowStat.numpackets_tcp);
-	printf("Packets_udp: %llu\n", NetflowStat.numpackets_udp);
-	printf("Packets_icmp: %llu\n", NetflowStat.numpackets_icmp);
-	printf("Packets_other: %llu\n", NetflowStat.numpackets_other);
-	printf("Bytes: %llu\n", NetflowStat.numbytes);
-	printf("Bytes_tcp: %llu\n", NetflowStat.numbytes_tcp);
-	printf("Bytes_udp: %llu\n", NetflowStat.numbytes_udp);
-	printf("Bytes_icmp: %llu\n", NetflowStat.numbytes_icmp);
-	printf("Bytes_other: %llu\n", NetflowStat.numbytes_other);
-	printf("First: %u\n", NetflowStat.first_seen);
-	printf("Last: %u\n", NetflowStat.last_seen);
-	printf("msec_first: %u\n", NetflowStat.msec_first);
-	printf("msec_last: %u\n", NetflowStat.msec_last);
-*/
-	CurrentIdent		= FileHeader.ident;
+	CurrentIdent		= nffile->file_header->ident;
 
-	if ( file_compressed && !lzo_initialized && !LZO_initialize() ) {
-		*err = error_string;
-		ZeroStat();
-		close(fd);
-		return -1;
+	if ( FILE_IS_COMPRESSED(nffile) && !lzo_initialized && !LZO_initialize() ) {
+		if ( allocated ) {
+			DisposeFile(nffile);
+			return NULL;
+		}
     }
 
-	return fd;
+	return nffile;
 
 } // End of OpenFile
 
-int ChangeIdent(char *filename, char *Ident, char **err) {
+void CloseFile(nffile_t *nffile){
+
+	if ( !nffile ) 
+		return;
+
+	// do not close stdout
+	if ( nffile->fd )
+		close(nffile->fd);
+
+} // End of CloseFile
+
+int ChangeIdent(char *filename, char *Ident) {
+file_header_t	FileHeader;
 struct stat stat_buf;
 int fd, ret;
 
-	*err = NULL;
 	if ( filename == NULL ) 
 		return 0;
 
 	if ( stat(filename, &stat_buf) ) {
-		snprintf(error_string, ERR_SIZE, "Can't stat '%s': %s\n", filename, strerror(errno));
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+		LogError("Can't stat '%s': %s\n", filename, strerror(errno));
 		return -1;
 	}
 
 	if (!S_ISREG(stat_buf.st_mode) ) {
-		snprintf(error_string, ERR_SIZE, "'%s' is not a file\n", filename);
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+		LogError("'%s' is not a file\n", filename);
 		return -1;
 	}
 
 	fd =  open(filename, O_RDWR);
 	if ( fd < 0 ) {
-		snprintf(error_string, ERR_SIZE, "Error open file: %s\n", strerror(errno));
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+		LogError("Error open file: %s\n", strerror(errno));
 		return fd;
 	}
 
 	ret = read(fd, (void *)&FileHeader, sizeof(FileHeader));
 	if ( FileHeader.magic != MAGIC ) {
-		snprintf(error_string, ERR_SIZE, "Open file '%s': bad magic: 0x%X\n", filename, FileHeader.magic );
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+		LogError("Open file '%s': bad magic: 0x%X\n", filename, FileHeader.magic );
 		close(fd);
 		return -1;
 	}
 	if ( FileHeader.version != LAYOUT_VERSION_1 ) {
-		snprintf(error_string, ERR_SIZE,"Open file %s: bad version: %u\n", filename, FileHeader.version );
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+		LogError("Open file %s: bad version: %u\n", filename, FileHeader.version );
 		close(fd);
 		return -1;
 	}
 
-	strncpy(FileHeader.ident, Ident, IdentLen);
-	FileHeader.ident[IdentLen - 1] = 0;
+	strncpy(FileHeader.ident, Ident, IDENTLEN);
+	FileHeader.ident[IDENTLEN - 1] = 0;
 
 	if ( lseek(fd, 0, SEEK_SET) < 0 ) {
-		snprintf(error_string, ERR_SIZE,"lseek failed: '%s'\n" , strerror(errno));
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+		LogError("lseek() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		close(fd);
 		return -1;
 	}
 
-	write(fd, (void *)&FileHeader, sizeof(file_header_t));
+	if ( write(fd, (void *)&FileHeader, sizeof(file_header_t)) <= 0 ) {
+		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+	}
+
 	if ( close(fd) < 0 ) {
-		snprintf(error_string, ERR_SIZE,"close failed: '%s'" , strerror(errno));
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+		LogError("close() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		return -1;
 	}
 	
@@ -352,11 +309,11 @@ int fd, ret;
 void PrintStat(stat_record_t *s) {
 
 	if ( s == NULL )
-		s = &NetflowStat;
+		return;
 
 	// format info: make compiler happy with conversion to (unsigned long long), 
 	// which does not change the size of the parameter
-	printf("Ident: %s\n", FileHeader.ident);
+	printf("Ident: %s\n", CurrentIdent);
 	printf("Flows: %llu\n", (unsigned long long)s->numflows);
 	printf("Flows_tcp: %llu\n", (unsigned long long)s->numflows_tcp);
 	printf("Flows_udp: %llu\n", (unsigned long long)s->numflows_udp);
@@ -379,67 +336,62 @@ void PrintStat(stat_record_t *s) {
 	printf("Sequence failures: %u\n", s->sequence_failure);
 } // End of PrintStat
 
-static void InitFile(nffile_t *nffile) {
-
-	// Init header
-	nffile->file_header->magic 	   = MAGIC;
-	nffile->file_header->version   = LAYOUT_VERSION_1;
-	nffile->file_header->flags 	   = 0;
-	nffile->file_header->NumBlocks = 0;
-
-	// Init vars
-	nffile->writeto		 = NULL;
-	nffile->wfd			 = 0;
-
-	// Init block header
-	nffile->block_header->size 		 = 0;
-	nffile->block_header->NumRecords = 0;
-	nffile->block_header->id		 = DATA_BLOCK_TYPE_2;
-	nffile->block_header->pad		 = 0;
-	nffile->writeto = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
-
-} // End of InitFile
-
-nffile_t *NewFile(void) {
+static nffile_t *NewFile(void) {
 nffile_t *nffile;
 
 	// Create struct
 	nffile = calloc(1, sizeof(nffile_t));
 	if ( !nffile ) {
-		snprintf(error_string, ERR_SIZE, "malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		error_string[ERR_SIZE-1] = 0;
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		return NULL;
 	}
+	nffile->buff_ptr = NULL;
+	nffile->fd	 	= 0;
 
 	// Init file header
 	nffile->file_header = calloc(1, sizeof(file_header_t));
 	if ( !nffile->file_header ) {
-		snprintf(error_string, ERR_SIZE, "malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		error_string[ERR_SIZE-1] = 0;
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		return NULL;
+	}
+	nffile->file_header->magic 	   = MAGIC;
+	nffile->file_header->version   = LAYOUT_VERSION_1;
+	nffile->file_header->flags 	   = 0;
+	nffile->file_header->NumBlocks = 0;
+
+	nffile->stat_record = calloc(1, sizeof(stat_record_t));
+	if ( !nffile->stat_record ) {
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		return NULL;
 	}
 
-	// init output data buffer
+	// init data buffer
 	nffile->block_header = malloc(BUFFSIZE + sizeof(data_block_header_t));
 	if ( !nffile->block_header ) {
-		snprintf(error_string, ERR_SIZE, "malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		error_string[ERR_SIZE-1] = 0;
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		return NULL;
 	}
+	nffile->block_header->size 		 = 0;
+	nffile->block_header->NumRecords = 0;
+	nffile->block_header->id		 = DATA_BLOCK_TYPE_2;
+	nffile->block_header->pad		 = 0;
 
+	nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
+	
 	return nffile;
 
 } // End of NewFile
 
 nffile_t *DisposeFile(nffile_t *nffile) {
 	free(nffile->file_header);
-	free(nffile->block_header);
+	free(nffile->stat_record);
+	if (nffile->block_header) 
+		free(nffile->block_header);
 	free(nffile);
 	return NULL;
-} // End of 
+} // End of DisposeFile
 
-nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int compressed, int anonymized, char **err) {
-stat_record_t	stat_record;
+nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int compressed, int anonymized, char *ident) {
 size_t			len;
 int 			flags;
 
@@ -447,13 +399,10 @@ int 			flags;
 	if ( nffile == NULL ) {
 		nffile = NewFile();
 		if ( nffile == NULL ) {
-			*err = error_string;
 			return NULL;
 		}
 	}
 
-	InitFile(nffile);
-
 	flags = compressed ? FLAG_COMPRESSED : 0;
 	if ( anonymized ) 
 		SetFlag(flags, FLAG_ANONYMIZED);
@@ -461,44 +410,46 @@ int 			flags;
 	nffile->file_header->flags 	   = flags;
 
 	if ( strcmp(filename, "-") == 0 ) { // output to stdout
-		nffile->wfd = STDOUT_FILENO;
+		nffile->fd = STDOUT_FILENO;
 	} else {
-		nffile->wfd = open(filename, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
-		if ( nffile->wfd < 0 ) {
-			snprintf(error_string, ERR_SIZE, "Failed to open file %s: '%s'" , filename, strerror(errno));
-			error_string[ERR_SIZE-1] = 0;
-			*err = error_string;
+		nffile->fd = open(filename, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
+		if ( nffile->fd < 0 ) {
+			LogError("Failed to open file %s: '%s'" , filename, strerror(errno));
 			return NULL;
 		}
 	}
 
-	memset((void *)&stat_record, 0, sizeof(stat_record));
+	memset((void *)nffile->stat_record, 0, sizeof(stat_record_t));
+	nffile->stat_record->first_seen = 0x7fffffff;
+	nffile->stat_record->msec_first = 999;
+
+	if ( ident ) {
+		strncpy(nffile->file_header->ident, ident, IDENTLEN);
+		nffile->file_header->ident[IDENTLEN - 1] = 0;
+	} 
+
 
 	if ( TestFlag(flags, FLAG_COMPRESSED) ) {
 		if ( !lzo_initialized && !LZO_initialize() ) {
-			snprintf(error_string, ERR_SIZE, "Failed to initialize compression");
-			*err = error_string;
-			close(nffile->wfd);
+			LogError("Failed to initialize compression");
+			close(nffile->fd);
 			return NULL;
 		}
-    }
+    	}
 
+	nffile->file_header->NumBlocks = 0;
 	len = sizeof(file_header_t);
-	if ( write(nffile->wfd, (void *)nffile->file_header, len) < len ) {
-		snprintf(error_string, ERR_SIZE, "Failed to write file header: '%s'" , strerror(errno));
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
-		close(nffile->wfd);
+	if ( write(nffile->fd, (void *)nffile->file_header, len) < len ) {
+		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(nffile->fd);
 		return NULL;
 	}
 
 	// write empty stat record - ist updated when file gets closed
 	len = sizeof(stat_record_t);
-	if ( write(nffile->wfd, (void *)&stat_record, len) < len ) {
-		snprintf(error_string, ERR_SIZE, "Failed to write file header: '%s'" , strerror(errno));
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
-		close(nffile->wfd);
+	if ( write(nffile->fd, (void *)nffile->stat_record, len) < len ) {
+		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(nffile->fd);
 		return NULL;
 	}
 
@@ -506,158 +457,231 @@ int 			flags;
 
 } /* End of OpenNewFile */
 
-void CloseUpdateFile(nffile_t *nffile, stat_record_t *stat_record, char *ident, char **err ) {
+nffile_t *AppendFile(char *filename) {
+nffile_t		*nffile;
+
+	// try to open the existing file
+	nffile = OpenFile(filename, NULL);
+	if ( !nffile )
+		return NULL;
+
+	// file is valid - re-open the file mode RDWR
+	close(nffile->fd);
+	nffile->fd = open(filename, O_RDWR | O_APPEND, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
+	if ( nffile->fd < 0 ) {
+		LogError("Failed to open file %s: '%s'" , filename, strerror(errno));
+		DisposeFile(nffile);
+		return NULL;
+	}
+
+	// init output data buffer
+	nffile->block_header = malloc(BUFFSIZE + sizeof(data_block_header_t));
+	if ( !nffile->block_header ) {
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(nffile->fd);
+		DisposeFile(nffile);
+		return NULL;
+	}
+	nffile->block_header->size 		 = 0;
+	nffile->block_header->NumRecords = 0;
+	nffile->block_header->id		 = DATA_BLOCK_TYPE_2;
+	nffile->block_header->pad		 = 0;
+	nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
+
+	// initialize output  lzo buffer
+	if ( FILE_IS_COMPRESSED(nffile) ) {
+		if ( !lzo_initialized && !LZO_initialize() ) {
+			LogError("Failed to initialize compression");
+			close(nffile->fd);
+			DisposeFile(nffile);
+			return NULL;
+		}
+    }
+
+	return nffile;
+
+} /* End of AppendFile */
+
+int CloseUpdateFile(nffile_t *nffile, char *ident) {
 file_header_t	file_header;
 
-	*err = NULL;
-	if ( lseek(nffile->wfd, 0, SEEK_SET) < 0 ) {
+	if ( nffile->block_header->size ) {
+		int ret = WriteBlock(nffile);
+		if ( ret < 0 ) {
+			LogError("Failed to flush output buffer");
+			return 0;
+		}
+	}
+
+	if ( lseek(nffile->fd, 0, SEEK_SET) < 0 ) {
 		// lseek on stdout works if output redirected:
 		// e.g. -w - > outfile
 		// but fails on pipe e.g. -w - | ./nfdump .... 
-		if ( nffile->wfd == STDOUT_FILENO ) {
-			return;
+		if ( nffile->fd == STDOUT_FILENO ) {
+			return 1;
 		} else {
-			snprintf(error_string, ERR_SIZE,"lseek failed: '%s'\n" , strerror(errno));
-			error_string[ERR_SIZE-1] = 0;
-			*err = error_string;
-			close(nffile->wfd);
-			return;
+			LogError("lseek() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			close(nffile->fd);
+			return 0;
 		}
 	}
 
-	strncpy(nffile->file_header->ident, ident ? ident : "unknown" , IdentLen);
-	file_header.ident[IdentLen - 1] = 0;
-
-	write(nffile->wfd, (void *)nffile->file_header, sizeof(file_header_t));
-	write(nffile->wfd, (void *)stat_record, sizeof(stat_record_t));
-	if ( close(nffile->wfd) < 0 ) {
-		snprintf(error_string, ERR_SIZE,"close failed: '%s'" , strerror(errno));
-		error_string[ERR_SIZE-1] = 0;
-		*err = error_string;
+	if ( ident ) {
+		strncpy(nffile->file_header->ident, ident, IDENTLEN);
+	} else {
+		if ( strlen(nffile->file_header->ident) == 0 ) 
+		strncpy(nffile->file_header->ident, IDENTNONE, IDENTLEN);
 	}
+	file_header.ident[IDENTLEN - 1] = 0;
+
+	if ( write(nffile->fd, (void *)nffile->file_header, sizeof(file_header_t)) <= 0 ) {
+		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+	}
+	if ( write(nffile->fd, (void *)nffile->stat_record, sizeof(stat_record_t)) <= 0 ) {
+		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+	}
+	if ( close(nffile->fd) < 0 ) {
+		LogError("close() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		return 0;
+	}
+
+	nffile->file_header->NumBlocks = 0;
 	
-	return;
+	return 1;
 
 } /* End of CloseUpdateFile */
 
-int ReadBlock(int rfd, data_block_header_t *block_header, void *read_buff, char **err) {
+int ReadBlock(nffile_t *nffile) {
 ssize_t ret, read_bytes, buff_bytes, request_size;
 void 	*read_ptr, *buff;
 
-		ret = read(rfd, block_header, sizeof(data_block_header_t));
-		if ( ret == 0 )		// EOF
-			return NF_EOF;
+	ret = read(nffile->fd, nffile->block_header, sizeof(data_block_header_t));
+	if ( ret == 0 )		// EOF
+		return NF_EOF;
 		
-		if ( ret == -1 )	// ERROR
-			return NF_ERROR;
+	if ( ret == -1 )	// ERROR
+		return NF_ERROR;
 		
-		// block header read successfully
-		read_bytes = ret;
+	// Check for sane buffer size
+	if ( ret != sizeof(data_block_header_t) ) {
+		// this is most likely a corrupt file
+		LogError("Corrupt data file: Read %i bytes, requested %u\n", ret, sizeof(data_block_header_t));
+		return NF_CORRUPT;
+	}
 
-		// Check for sane buffer size
-		if ( block_header->size > BUFFSIZE ) {
-			snprintf(error_string, ERR_SIZE, "Corrupt data file: Requested buffer size %u exceeds max. buffer size.\n", block_header->size);
-			error_string[ERR_SIZE-1] = 0;
-			*err = error_string;
-			// this is most likely a corrupt file
-			return NF_CORRUPT;
-		}
+	// block header read successfully
+	read_bytes = ret;
 
-		buff = file_compressed ? lzo_buff : read_buff;
+	// Check for sane buffer size
+	if ( nffile->block_header->size > BUFFSIZE ) {
+		// this is most likely a corrupt file
+		LogError("Corrupt data file: Requested buffer size %u exceeds max. buffer size.\n", nffile->block_header->size);
+		return NF_CORRUPT;
+	}
 
-		ret = read(rfd, buff, block_header->size);
-		if ( ret == block_header->size ) {
-			lzo_uint new_len;
-			// we have the whole record and are done for now
-			if ( file_compressed ) {
-				int r;
-    			r = lzo1x_decompress(lzo_buff,block_header->size,read_buff,&new_len,NULL);
-    			if (r != LZO_E_OK ) {
-        			/* this should NEVER happen */
-        			printf("internal error - decompression failed: %d\n", r);
-        			return NF_CORRUPT;
-    			}
-				block_header->size = new_len;
-				return read_bytes + new_len;
-			} else
-				return read_bytes + ret;
+	buff = FILE_IS_COMPRESSED(nffile) ? lzo_buff : nffile->buff_ptr;
 
-		} 
-			
-		if ( ret == 0 ) {
-			// EOF not expected here - this should never happen, file may be corrupt
-			snprintf(error_string, ERR_SIZE, "Corrupt data file: Unexpected EOF while reading data block.\n");
-			error_string[ERR_SIZE-1] = 0;
-			*err = error_string;
-			return NF_CORRUPT;
-		}
-
-		if ( ret == -1 )	// ERROR
-			return NF_ERROR;
-
-		// Ups! - ret is != block_header->size
-		// this was a short read - most likely reading from the stdin pipe
-		// loop until we have requested size
-
-		buff_bytes 	 = ret;								// already in buffer
-		request_size = block_header->size - buff_bytes;	// still to go for this amount of data
-
-		read_ptr 	 = (void *)((pointer_addr_t)buff + buff_bytes);	
-		do {
-
-			ret = read(rfd, read_ptr, request_size);
-			if ( ret < 0 ) 
-				// -1: Error - not expected
-				return NF_ERROR;
-
-			if ( ret == 0 ) {
-				//  0: EOF   - not expected
-				snprintf(error_string, ERR_SIZE, "Corrupt data file: Unexpected EOF. Short read of data block.\n");
-				error_string[ERR_SIZE-1] = 0;
-				*err = error_string;
-				return NF_CORRUPT;
-			} 
-			
-			buff_bytes 	 += ret;
-			request_size = block_header->size - buff_bytes;
-
-			if ( request_size > 0 ) {
-				// still a short read - continue in read loop
-				read_ptr 	 = (void *)((pointer_addr_t)buff + buff_bytes);
-			}
-		} while ( request_size > 0 );
-
-		if ( file_compressed ) {
+	ret = read(nffile->fd, buff, nffile->block_header->size);
+	if ( ret == nffile->block_header->size ) {
+		lzo_uint new_len;
+		// we have the whole record and are done for now
+		if ( FILE_IS_COMPRESSED(nffile) ) {
 			int r;
-			lzo_uint new_len;
-    		r = lzo1x_decompress(lzo_buff,block_header->size,read_buff,&new_len,NULL);
-    		if (r != LZO_E_OK ) {
-        		/* this should NEVER happen */
-        		printf("internal error - decompression failed: %d\n", r);
-        		return NF_CORRUPT;
-    		}
-			block_header->size = new_len;
+   			r = lzo1x_decompress(lzo_buff,nffile->block_header->size,nffile->buff_ptr,&new_len,NULL);
+   			if (r != LZO_E_OK ) {
+       			/* this should NEVER happen */
+				LogError("ReadBlock() error decompression failed in %s line %d: LZO error: %d\n", __FILE__, __LINE__, r);
+       			return NF_CORRUPT;
+   			}
+			nffile->block_header->size = new_len;
 			return read_bytes + new_len;
+		} else
+			return read_bytes + ret;
+	} 
+			
+	if ( ret == 0 ) {
+		// EOF not expected here - this should never happen, file may be corrupt
+		LogError("ReadBlock() Corrupt data file: Unexpected EOF while reading data block.\n");
+		return NF_CORRUPT;
+	}
 
-		} else {
-			// finally - we are done for now
-			return read_bytes + buff_bytes;
+	if ( ret == -1 ) {	// ERROR
+		LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		return NF_ERROR;
+	}
+
+	// Ups! - ret is != block_header->size
+	// this was a short read - most likely reading from the stdin pipe
+	// loop until we have requested size
+
+	buff_bytes 	 = ret;								// already in buffer
+	request_size = nffile->block_header->size - buff_bytes;	// still to go for this amount of data
+
+	read_ptr 	 = (void *)((pointer_addr_t)buff + buff_bytes);	
+	do {
+		ret = read(nffile->fd, read_ptr, request_size);
+		if ( ret < 0 ) 
+			// -1: Error - not expected
+			LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			return NF_ERROR;
+
+		if ( ret == 0 ) {
+			//  0: EOF   - not expected
+			LogError(error_string, ERR_SIZE, "Corrupt data file: Unexpected EOF. Short read of data block.\n");
+			return NF_CORRUPT;
+		} 
+		
+		buff_bytes 	 += ret;
+		request_size = nffile->block_header->size - buff_bytes;
+
+		if ( request_size > 0 ) {
+			// still a short read - continue in read loop
+			read_ptr = (void *)((pointer_addr_t)buff + buff_bytes);
 		}
-	
-		/* not reached */
+	} while ( request_size > 0 );
+
+	if ( FILE_IS_COMPRESSED(nffile) ) {
+		int r;
+		lzo_uint new_len;
+   		r = lzo1x_decompress(lzo_buff, nffile->block_header->size, nffile->buff_ptr, &new_len, NULL);
+   		if (r != LZO_E_OK ) {
+       		/* this should NEVER happen */
+			LogError("ReadBlock() error decompression failed in %s line %d: LZO error: %d\n", __FILE__, __LINE__, r);
+       		return NF_CORRUPT;
+   		}
+		nffile->block_header->size = new_len;
+		return read_bytes + new_len;
+
+	} else {
+		// finally - we are done for now
+		return read_bytes + buff_bytes;
+	}
+
+	/* not reached */
 
 } // End of ReadBlock
 
 int WriteBlock(nffile_t *nffile) {
 data_block_header_t *out_block_header;
-int r;
+int r, ret;
 unsigned char __LZO_MMODEL *in;
 unsigned char __LZO_MMODEL *out;
 lzo_uint in_len;
 lzo_uint out_len;
 
+	// empty blocks need not to be stored 
+	if ( nffile->block_header->size == 0 )
+		return 1;
+
 	if ( !TestFlag(nffile->file_header->flags, FLAG_COMPRESSED) ) {
-		return write(nffile->wfd, (void *)nffile->block_header, sizeof(data_block_header_t) + nffile->block_header->size);
+		ret = write(nffile->fd, (void *)nffile->block_header, sizeof(data_block_header_t) + nffile->block_header->size);
+		if ( ret > 0 ) {
+			nffile->block_header->size 		 = 0;
+			nffile->block_header->NumRecords = 0;
+			nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t) );
+			nffile->file_header->NumBlocks++;
+		}
+		return ret;
 	} 
 
 	out_block_header = (data_block_header_t *)lzo_buff;
@@ -675,10 +699,58 @@ lzo_uint out_len;
 	}
 
 	out_block_header->size = out_len;
-	return write(nffile->wfd, (void *)out_block_header, sizeof(data_block_header_t) + out_block_header->size);
+	ret = write(nffile->fd, (void *)out_block_header, sizeof(data_block_header_t) + out_block_header->size);
+	if ( ret > 0 ) {
+		nffile->block_header->size 		 = 0;
+		nffile->block_header->NumRecords = 0;
+		nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t) );
+		nffile->file_header->NumBlocks++;
+	}
+
+	return ret;
 
 } // End of WriteBlock
 
+int WriteExtraBlock(nffile_t *nffile, data_block_header_t *block_header) {
+data_block_header_t *out_block_header;
+int r, ret;
+unsigned char __LZO_MMODEL *in;
+unsigned char __LZO_MMODEL *out;
+lzo_uint in_len;
+lzo_uint out_len;
+
+	if ( !TestFlag(nffile->file_header->flags, FLAG_COMPRESSED) ) {
+		ret =  write(nffile->fd, (void *)block_header, sizeof(data_block_header_t) + block_header->size);
+		if ( ret > 0 ) {
+			nffile->file_header->NumBlocks++;
+		}
+		return ret;
+	} 
+
+	out_block_header = (data_block_header_t *)lzo_buff;
+	*out_block_header = *(block_header);
+
+	in  = (unsigned char __LZO_MMODEL *)((pointer_addr_t)block_header     + sizeof(data_block_header_t));	
+	out = (unsigned char __LZO_MMODEL *)((pointer_addr_t)out_block_header + sizeof(data_block_header_t));	
+	in_len = block_header->size;
+	r = lzo1x_1_compress(in,in_len,out,&out_len,wrkmem);
+
+	if (r != LZO_E_OK) {
+		snprintf(error_string, ERR_SIZE,"compression failed: %d" , r);
+		error_string[ERR_SIZE-1] = 0;
+		return -2;
+	}
+
+	out_block_header->size = out_len;
+	ret =  write(nffile->fd, (void *)out_block_header, sizeof(data_block_header_t) + out_block_header->size);
+	if ( ret > 0 ) {
+		nffile->file_header->NumBlocks++;
+	}
+
+	return ret;
+
+} // End of WriteExtraBlock
+
 
 inline void ExpandRecord_v1(common_record_t *input_record, master_record_t *output_record ) {
 uint32_t	*u;
@@ -738,17 +810,15 @@ void		*p = (void *)input_record;
 } // End of ExpandRecord_v1
 
 void UnCompressFile(char * filename) {
-int 			i, rfd, flags, compressed, anonymized;
+int 			i, flags, compressed, anonymized;
 ssize_t			ret;
-nffile_t		*nffile;
-stat_record_t 	*stat_ptr;
-char			*string;
+nffile_t		*nffile_r, *nffile_w;
+stat_record_t	*_s;
 char 			outfile[MAXPATHLEN];
-void			*buff_ptr;
+void			*tmp;
 
-	rfd = OpenFile(filename, &stat_ptr, &string);
-	if ( rfd < 0 ) {
-		fprintf(stderr, "%s\n", string);
+	nffile_r = OpenFile(filename, NULL);
+	if ( !nffile_r ) {
 		return;
 	}
 	
@@ -756,110 +826,213 @@ void			*buff_ptr;
 	snprintf(outfile, MAXPATHLEN, "%s-tmp", filename);
 	outfile[MAXPATHLEN-1] = '\0';
 
-	flags = FileHeader.flags;
-	if ( file_compressed ) {
-		printf("Uncompress file .. \n");
+	flags = nffile_r->file_header->flags;
+	if ( FILE_IS_COMPRESSED(nffile_r) ) {
+		printf("Uncompress file %s ..\n", filename);
 		compressed = 0;
 	} else {
-		printf("Compress file .. \n");
+		printf("Compress file %s .. \n", filename);
 		compressed = 1;
 	}
-	anonymized = IsAnonymized();
+	anonymized = IP_ANONYMIZED(nffile_r);
 
 	// allocate output file
-	nffile = OpenNewFile(outfile, NULL, compressed, anonymized, &string);
-	if ( !nffile ) {
-		fprintf(stderr, "%s\n", string);
-		close(rfd);
+	nffile_w = OpenNewFile(outfile, NULL, compressed, anonymized, NULL);
+	if ( !nffile_w ) {
+		CloseFile(nffile_r);
+		DisposeFile(nffile_r);
 		return;
 	}
 
-	buff_ptr	 = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
+	// Use same buffer for read/write
+	tmp = nffile_r->block_header;
+	nffile_r->block_header =  nffile_w->block_header;
+	nffile_r->buff_ptr 	   =  nffile_w->buff_ptr;
 
-	for ( i=0; i < FileHeader.NumBlocks; i++ ) {
-		ret = ReadBlock(rfd, nffile->block_header, buff_ptr, &string);
+	// swap stat records :)
+	_s = nffile_r->stat_record;
+	nffile_r->stat_record = nffile_w->stat_record;
+	nffile_w->stat_record = _s;
+
+	for ( i=0; i < nffile_r->file_header->NumBlocks; i++ ) {
+		ret = ReadBlock(nffile_r);
 		if ( ret < 0 ) {
-			fprintf(stderr, "Error while reading data block. Abort.\n");
-			close(rfd);
-			close(nffile->wfd);
+			LogError("Error while reading data block. Abort.\n");
+			nffile_r->block_header = tmp;
+			CloseFile(nffile_r);
+			DisposeFile(nffile_r);
+			CloseFile(nffile_w);
+			DisposeFile(nffile_w);
 			unlink(outfile);
 			return;
 		}
-		if ( WriteBlock(nffile) <= 0 ) {
-			fprintf(stderr, "Failed to write output buffer to disk: '%s'" , strerror(errno));
-			close(rfd);
-			close(nffile->wfd);
+		if ( WriteBlock(nffile_w) <= 0 ) {
+			LogError("Failed to write output buffer to disk: '%s'" , strerror(errno));
+			nffile_r->block_header = tmp;
+			CloseFile(nffile_r);
+			DisposeFile(nffile_r);
+			CloseFile(nffile_w);
+			DisposeFile(nffile_w);
 			unlink(outfile);
 			return;
 		}
 	}
 
-	close(rfd);
-	CloseUpdateFile(nffile, stat_ptr, GetIdent(), &string );
-	if ( string != NULL ) {
-		fprintf(stderr, "%s\n", string);
-		close(nffile->wfd);
+	// file processed 
+	nffile_r->block_header = tmp;
+	CloseFile(nffile_r);
+
+	if ( !CloseUpdateFile(nffile_w, nffile_r->file_header->ident) ) {
 		unlink(outfile);
+		LogError("Failed to close file: '%s'" , strerror(errno));
 	} else {
-		close(nffile->wfd);
 		unlink(filename);
 		rename(outfile, filename);
 	}
-	DisposeFile(nffile);
+
+	DisposeFile(nffile_r);
+	DisposeFile(nffile_w);
 
 } // End of UnCompressFile
 
 void QueryFile(char *filename) {
-int i, fd;
-stat_record_t *stat_ptr;
-data_block_header_t block_header;
-char	*string;
-uint32_t num_records, type1, type2;
+int i;
+nffile_t	*nffile;
+uint32_t num_records, type1, type2, type3;
+struct stat stat_buf;
 ssize_t	ret;
+off_t	fsize;
 
-	fd = OpenFile(filename, &stat_ptr, &string);
-	if ( fd < 0 ) {
-		fprintf(stderr, "%s\n", string);
+	if ( stat(filename, &stat_buf) ) {
+		LogError("Can't stat '%s': %s\n", filename, strerror(errno));
+		return;
+	}
+
+	nffile = OpenFile(filename, NULL);
+	if ( !nffile ) {
 		return;
 	}
 
 	num_records = 0;
+	// set file size to current position ( file header )
+	fsize = lseek(nffile->fd, 0, SEEK_CUR);
 	type1 = 0;
 	type2 = 0;
+	type3 = 0;
 	printf("File    : %s\n", filename);
-	printf("Version : %u - %s\n", FileHeader.version, file_compressed ? "compressed" : "not compressed");
-	printf("Blocks  : %u\n", FileHeader.NumBlocks);
-	for ( i=0; i < FileHeader.NumBlocks; i++ ) {
-		ret = read(fd, (void *)&block_header, sizeof(data_block_header_t));
-		if ( ret < 0 ) {
-			fprintf(stderr, "Error reading block %i: %s\n", i, strerror(errno));
-			return;
+	printf("Version : %u - %s\n", nffile->file_header->version, FILE_IS_COMPRESSED(nffile) ? "compressed" : "not compressed");
+	printf("Blocks  : %u\n", nffile->file_header->NumBlocks);
+	for ( i=0; i < nffile->file_header->NumBlocks; i++ ) {
+		if ( (fsize + sizeof(data_block_header_t)) > stat_buf.st_size ) {
+			LogError("Unexpected read beyond EOF! File corrupted. Abort.\n");
+			LogError("Expected %u blocks, counted %i\n", nffile->file_header->NumBlocks, i);
+			break;
 		}
-		num_records += block_header.NumRecords;
-		switch ( block_header.id) {
+		ret = read(nffile->fd, (void *)nffile->block_header, sizeof(data_block_header_t));
+		if ( ret < 0 ) {
+			LogError("Error reading block %i: %s\n", i, strerror(errno));
+			break;
+		}
+
+		// Should never happen, as catched already in first check, but test it anyway ..
+		if ( ret == 0 ) {
+			LogError("Unexpected end of file reached. Expected %u blocks, counted %i\n", nffile->file_header->NumBlocks, i);
+			break;
+		}
+		if ( ret < sizeof(data_block_header_t) ) {
+			LogError("Short read: Expected %u bytes, read: %i\n", sizeof(data_block_header_t), ret);
+			break;
+		}
+		fsize += sizeof(data_block_header_t);
+
+		num_records += nffile->block_header->NumRecords;
+		switch ( nffile->block_header->id) {
 			case DATA_BLOCK_TYPE_1:
 				type1++;
 				break;
 			case DATA_BLOCK_TYPE_2:
 				type2++;
 				break;
+			case Large_BLOCK_Type:
+				type3++;
+				break;
 			default:
-				printf("block %i has unknown type %u\n", i, block_header.id);
+				printf("block %i has unknown type %u\n", i, nffile->block_header->id);
 		}
 
-		if ( lseek(fd, block_header.size, SEEK_CUR) < 0 ) {
-			fprintf(stderr, "Error seeking block %i: %s\n", i, strerror(errno));
-			return;
+		if ( (fsize + nffile->block_header->size ) > stat_buf.st_size ) {
+			LogError("Expected to seek beyond EOF! File corrupted. Abort.\n");
+			break;
+		}
+		fsize += nffile->block_header->size;
+		
+		ret = lseek(nffile->fd, nffile->block_header->size, SEEK_CUR);
+		if ( ret < 0 ) {
+			LogError("Error seeking block %i: %s\n", i, strerror(errno));
+			break;
+		}
+		if ( fsize != ret ) {
+			LogError("Expected seek: Expected: %u, got: %u\n", fsize, ret);
+			break;
 		}
 	}
+
+	if ( fsize < stat_buf.st_size ) {
+		LogError("Extra data detected after regular blocks: %i bytes\n", stat_buf.st_size-fsize);
+	}
+
 	printf(" Type 1 : %u\n", type1);
 	printf(" Type 2 : %u\n", type2);
+	printf(" Type 3 : %u\n", type3);
 	printf("Records : %u\n", num_records);
 
-	close(fd);
+	CloseFile(nffile);
+	DisposeFile(nffile);
 
 } // End of QueryFile
 
+// simple interface to get a statrecord from a file without nffile overhead
+stat_record_t *GetStatRecord(char *filename, stat_record_t *stat_record) {
+file_header_t file_header;
+int fd, ret;
+
+	fd = open(filename, O_RDONLY);
+	if ( fd < 0 ) {
+		LogError("open() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		return NULL;
+	}
+
+	ret = read(fd, (void *)&file_header, sizeof(file_header_t));
+	if ( ret < 0 ) {
+		LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(fd);
+		return NULL;
+	}
+
+	if ( file_header.magic != MAGIC ) {
+		LogError("Open file '%s': bad magic: 0x%X\n", filename ? filename : "<stdin>", file_header.magic );
+		close(fd);
+		return NULL;
+	}
+
+	if ( file_header.version != LAYOUT_VERSION_1 ) {
+		LogError("Open file %s: bad version: %u\n", filename, file_header.version );
+		close(fd);
+		return NULL;
+	}
+
+	ret = read(fd, (void *)stat_record, sizeof(stat_record_t));
+	if ( ret < 0 ) {
+		LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(fd);
+		return NULL;
+	}
+
+	close(fd);
+	return stat_record;
+
+} // End of GetStatRecord
+
 #ifdef COMPAT15
 /*
  * v1 -> v2 record conversion:
