@@ -178,6 +178,10 @@ int	v1_map_done = 0;
 		LogError("GetNextFile() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		return;
 	}
+	if ( nffile == EMPTY_LIST ) {
+		LogError("Empty file list. No files to process\n");
+		return;
+	}
 
 	done = 0;
 	while ( !done ) {
